/*
 * ls.c
 *  
 * Mark Claypool, WPI
 * 2011, 2016, 2017
 * 
 * Show basic directory operations.
 * Do "man readdir* for more information.
 */

#include <stdio.h>		/* For printf(), perror() */
#include <sys/types.h>		/* For opendir() */
#include <unistd.h>		/* For exit() */
#include <dirent.h>		/* For readdir(), opendir() */
#include <errno.h>		/* For errno */

/* (shown here for reference)
struct  dirent {
        ino_t    d_ino;               * inode number of entry *
        ushort_t d_reclen;            * length of this record *
        ushort_t d_namlen;            * length of string in d_name *
        char    d_name[256];	      * the actual directory names *
};
*/

int main() {

  /* Open directory process is in (current working directory). */
  DIR *dp = opendir(".");
  if (dp == NULL) {  /* error */
    perror("open()");
    return -1;			
  }

  /* Loop, reading each file and printing until done all. */
  struct dirent *d = readdir(dp);
  while (d) {
    printf("%s\n", d->d_name);
    d = readdir(dp);
  }

  /* Check if latest read was an error. */
  if (errno) {
    perror("readdir()");
    return -1;
  }

  /* Done, so close directory. */
  if (closedir(dp) != 0)  {  /* Error. */
    perror("closedir()");
    return -1;
  }

  /* All is well. */
  return 0;	
}
