/*
 * pipe.c - demonstrate basic pipe.
 */

#include <stdio.h>
#include <unistd.h>
#include <string.h>

#define STRING "Hello, world!"
#define STRING_MAX 80

int main(void) {

  /* Create a pipe, for 1-way communication between processes. */
  int fd[2];
  if (pipe(fd) == -1 ) {
    perror("pipe");
    return 1;
  }

  /* Create second process to communicate with. */
  int pid = fork();
  if (pid == -1) {
    perror("fork");
    return 1;
  }

  if (pid != 0) {  /** Parent. **/

    /* Close up input side of pipe. */
    if (close(fd[0]))
      perror("close");

    /* Send STRING through output side of pipe. */
    if (write(fd[1], STRING, strlen(STRING)+1) == -1)
      perror("write");
    else
      printf("Parent sent string: %s\n", STRING);

  } else {  /** Child. **/

    /* Close up output side of pipe. */
    if (close(fd[1]))
      perror("close");

    /* Read in string from pipe. */
    char buff[STRING_MAX];
    if (read(fd[0], buff, STRING_MAX) == -1)
      perror("read");
    else
      printf("Child received string: %s\n", buff);
  }

  /* All is well. */
  return 0;
}
