/*
 * shmem.c - demonstrate basic use of shared memory.
 */

#include <string.h>
#include <unistd.h>
#include <sys/mman.h>
#include <stdio.h>

#define HELLO "Hello,"
#define WORLD "world!"

int main(void) {

  /* Create shared memory segment. */
  int protect = PROT_READ | PROT_WRITE;        /* Read & write. */
  int visibile = MAP_ANONYMOUS | MAP_SHARED;   /* Shared, but anonymous. */
  int size = 100;                              /* 100 bytes. */
  char* shmem = (char *)                       /* NULL - don't care where. */
    mmap(NULL, size, protect, visibile, 0, 0); /* (0,0) don't init.*/
  if (shmem == MAP_FAILED) {
    perror("mmap");
    return 1;
  }
  
  /* Write parent message. */
  memcpy(shmem, HELLO, strlen(HELLO)+1);
  printf("Parent said: %s\n", shmem);

  /* Create second process to communicate with. */
  int pid = fork();
  if (pid == -1) {
    perror("fork");
    return 1;
  }

  if (pid == 0) {  /** Child. **/
    printf("Child heard: %s\n", shmem);
    memcpy(shmem, WORLD, strlen(WORLD)+1);
    printf("Child said: %s\n", shmem);
  } else {        /** Parent. **/
    sleep(1);
    printf("Parent heard: %s\n", shmem);
  }

  /* Remove shared memory segment. */
  if (munmap(shmem, 100) == -1) {
    perror("munmap");
    return 1;
  }

  /* All is well. */
  return 0;
}
