//
// Linux kernel version 4.4.0-83
// 
// From: /usr/src/linux-headers-4.4.0-83/include/linux/sched.h
// 
// Note:
// - Removed all #ifdef dependencies.
// - Re-arranged order of some attributes for discussion.
//

struct task_struct {

  // State stuff.
  volatile long state;	/* -1 unrunnable, 0 runnable, >0 stopped */
  void *stack;
  atomic_t usage;
  unsigned int flags;  /* per process flags, defined below */
  unsigned int ptrace;

  // PID and such.
  pid_t pid;
  pid_t tgid;
  struct pid_link pids[PIDTYPE_MAX];
  struct list_head thread_group;
  struct list_head thread_node;

  // All processes/tasks (PCB table). 
  struct list_head tasks; 
  /*
   * pointers to (original) parent process, youngest child, younger sibling,
   * older sibling, respectively.
   */
  struct task_struct __rcu *real_parent; /* real parent process */
  struct task_struct __rcu *parent; /* recipient of SIGCHLD, wait4() reports */
  // children/sibling forms the list of my natural children
  struct list_head children;  /* list of my children */
  struct list_head sibling;  /* linkage in my parent's children list */
  struct task_struct *group_leader;  /* threadgroup leader */

  // Used for scheduling.
  int prio, static_prio, normal_prio;
  unsigned int rt_priority;
  const struct sched_class *sched_class;
  struct sched_entity se;
  struct sched_rt_entity rt;
  struct sched_dl_entity dl;
  unsigned int policy;
  int nr_cpus_allowed;
  cpumask_t cpus_allowed;

  int on_rq;

  // Used for memory management.
  struct mm_struct *mm, *active_mm;
  u32 vmacache_seqnum;
  struct vm_area_struct *vmacache[VMACACHE_SIZE];

  // Process/task state upon termination.
  int exit_state;
  int exit_code, exit_signal;
  int pdeath_signal;  /*  The signal sent when the parent dies  */
  unsigned long jobctl;  /* JOBCTL_*, siglock protected */

  // Used for emulating ABI behavior of previous Linux versions
  unsigned int personality;

  // scheduler bits, serialized by scheduler locks 
  unsigned sched_reset_on_fork:1;
  unsigned sched_contributes_to_load:1;
  unsigned sched_migrated:1;
  unsigned :0; /* force alignment to the next boundary */

  // Unserialized, strictly 'current'.
  unsigned in_execve:1; /* bit to tell LSMs we're in execve */
  unsigned in_iowait:1;

  unsigned long atomic_flags; /* Flags needing atomic access. */

  struct restart_block restart_block;

  /*
   * ptraced is the list of tasks this task is using ptrace on.
   * This includes both natural children and PTRACE_ATTACH targets.
   * p->ptrace_entry is p's link on the p->parent->ptraced list.
   */
  struct list_head ptraced;
  struct list_head ptrace_entry;

  struct completion *vfork_done;    /* for vfork() */
  int __user *set_child_tid;    /* CLONE_CHILD_SETTID */
  int __user *clear_child_tid;    /* CLONE_CHILD_CLEARTID */

  cputime_t utime, stime, utimescaled, stimescaled;
  cputime_t gtime;
  struct prev_cputime prev_cputime;

  unsigned long nvcsw, nivcsw; /* context switch counts */
  u64 start_time;    /* monotonic time in nsec */
  u64 real_start_time;  /* boot based time in nsec */
/* mm fault and swap info: this can arguably be seen as either mm-specific or thread-specific */
  unsigned long min_flt, maj_flt;

  struct task_cputime cputime_expires;
  struct list_head cpu_timers[3];

/* process credentials */
  const struct cred __rcu *ptracer_cred; /* Tracer's credentials at attach */
  const struct cred __rcu *real_cred; /* objective and real subjective task
           * credentials (COW) */
  const struct cred __rcu *cred;  /* effective (overridable) subjective task
           * credentials (COW) */
  char comm[TASK_COMM_LEN]; /* executable name excluding path
             - access with [gs]et_task_comm (which lock
               it with task_lock())
             - initialized normally by setup_new_exec */
/* file system info */
  struct nameidata *nameidata;
  struct fs_struct *fs;
  struct files_struct *files;
/* namespaces */
  struct nsproxy *nsproxy;
/* signal handlers */
  struct signal_struct *signal;
  struct sighand_struct *sighand;

  sigset_t blocked, real_blocked;
  sigset_t saved_sigmask;  /* restored if set_restore_sigmask() was used */
  struct sigpending pending;

  unsigned long sas_ss_sp;
  size_t sas_ss_size;

  struct callback_head *task_works;

  struct audit_context *audit_context;
  struct seccomp seccomp;

/* Thread group tracking */
     u32 parent_exec_id;
     u32 self_exec_id;
/* Protection of (de-)allocation: mm, files, fs, tty, keyrings, mems_allowed,
 * mempolicy */
  spinlock_t alloc_lock;

  /* Protection of the PI data structures: */
  raw_spinlock_t pi_lock;

  struct wake_q_node wake_q;

/* journalling filesystem info */
  void *journal_info;

/* stacked block device info */
  struct bio_list *bio_list;

/* VM state */
  struct reclaim_state *reclaim_state;

  struct backing_dev_info *backing_dev_info;

  struct io_context *io_context;

  unsigned long ptrace_message;
  siginfo_t *last_siginfo; /* For ptrace use.  */
  struct task_io_accounting ioac;


  struct rcu_head rcu;

  /*
   * cache last used pipe for splice
   */
  struct pipe_inode_info *splice_pipe;

  struct page_frag task_frag;

  /*
   * when (nr_dirtied >= nr_dirtied_pause), it's time to call
   * balance_dirty_pages() for some dirty throttling pause
   */
  int nr_dirtied;
  int nr_dirtied_pause;
  unsigned long dirty_paused_when; /* start of a write-and-pause period */

  /*
   * time slack values; these are used to round up poll() and
   * select() etc timeout values. These are in nanoseconds.
   */
  unsigned long timer_slack_ns;
  unsigned long default_timer_slack_ns;

  int pagefault_disabled;
/* CPU-specific state of this task */
  struct thread_struct thread;
/*
 * WARNING: on x86, 'thread_struct' contains a variable-sized
 * structure.  It *MUST* be at the end of 'task_struct'.
 *
 * Do not put anything below here!
 */
};
