/*
 * shell-v1.c - a basic shell
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>

#define MAX_INPUT 75

int main() {

  /* for user input */
  char input[MAX_INPUT];
  char *arg;

  /* for exec() */
  char *new_argv[5];
  int new_argc;

  while (1) {

    /* get input and remove newline */
    printf("==>> ");
    fgets(input, MAX_INPUT, stdin);
    input[strlen(input)-1] = '\0';
    
    /* parse input into initial command */
    arg = strtok(input, " ");
    if(!arg)
      continue; 

    /* built-in command 'exit' */
    if (strcmp(arg, "exit") == 0)
      return 0;

    /* built-in command 'cd' */
    if (strcmp(arg, "cd") == 0) {
      arg = strtok(NULL, " ");
      if (arg == NULL) {
	printf("cd <directory>\n");
	continue;
      }
      if (chdir(arg) == -1) {
	perror("Unable to change directory");
	continue;
      }
      continue;
    }

    /* fork child process */
    if (fork() == 0) {

      /** I am the child **/

      /* build command and args */
      new_argv[0] = arg;
      new_argc = 1;
      while (arg = strtok(NULL, " ")) {
	new_argv[new_argc] = arg;
	new_argc++;
      }
      new_argv[new_argc] = NULL;

      /* execute new command and check for error */
      /* (v - use argv array, p - use path) */
      if (-1 == execvp(new_argv[0], new_argv)) {
	printf("Unable to execute \"%s\"\n", new_argv[0]);
	return 1;
      }

    } else {

      /** I am the parent **/

      /* wait for any child process to finish */
      wait(NULL);

    }

  } /* while */

}
