/* Modify PCB to support threads. */

/*** Modify ***/

/* The process control block is the data structure that records the
/* state of the process.  It includes the register state and the
/* process state. */
struct ProcessControlBlock {
  int     threads;            /* a circular list of threads */
  bool    allocated;	      /* true if valid process, else false */
  void   *base, *bound;       /* for thread memory limits */
};

/*** Add ***/

/* Thread state enumerates all possible states for process. */
enum ThreadState { READY, RUNNING, BLOCKED };

/* The thread control block is the data structure that records the
/* state of each thread. */
struct ThreadControlBlock {
  ThreadState state;          /* Each thread has its own state */
  SaveArea sa;                /* Each thread has its own register save area */
  int     pid;                /* Process index this thread runs in. */
  int     timeLeft;	      /* Time left since last time slice (ms) */
  bool    allocated;	      /* True if valid thread, else false */
  int nextThread, prevThread; /* List of threads linked by indexes. */
};

/* OS global data. */

/* Process table and thread table. */
ProcessControlBlock pcb[MAX_PROCESSES];
ThreadControlBlock  tcb[MAX_THREADS];

int current_thread;  /* Thread that is currently running. */
