/*
 * pcb.h - Manage Process Control Block information (i.e., information
 *         required to manage running programs). 
 */

/* A save area allocates space to save all the hardware registers. */
struct SaveArea {
  int pc;		      /* program counter */
  int psw;		      /* program status word */
  int base;		      /* base address in memory */
  int bound;		      /* upper limit on address */
  int reg[NUM_REGISTERS];     /* register values */
};

/* Process state enumerates all possible states for process. */
enum ProcessState { READY, RUNNING, BLOCKED };

/* The process control block is the data structure that records the
/* state of the process.  It includes the register state and the
/* process state. */
struct ProcessControlBlock {
  struct  SaveArea sa;	      /* hardware state to save */
  enum    ProcessState state; /* ready, running or blocked */
  int     timeLeft;	      /* time left since last time slice (ms) */
  bool    allocated;	      /* true if valid process, else false */
};
