//
// Xv6 - a simple Unix-like teaching operating system.
// https://pdos.csail.mit.edu/6.828/2016/xv6.html
//

// Registers Xv6 saves and restores to stop and restart a process.
struct context {
  int eip;
  int esp;
  int ebx;
  int ecx;
  int edx;
  int esi;
  int edi;
  int ebp;
};

// Different states Xv6 processes can be in.
enum proc_state { UNUSED, EMBRYO, SLEEPING, RUNNABLE, RUNNING, ZOMBIE };

// Information Xv6 tracks about each process.
struct proc {
  enum proc_state state;  // Process state
  int pid;		  // Process ID
  struct proc *parent;	  // Parent process
  char *mem;              // Start of process memory
  uint sz;		  // Size of process memory
  char *kstack;		  // Bottom of kernel stack for this process
  struct file *ofile[NOFILE]; // Open files
  struct inode *cwd;	  // Current directory
  struct trapframe *tf;	  // Trap frame for the current interrupt
  struct context context; // Switch here to run process
  void *chan;		  // If non-zero, sleeping on channel
  int killed;		  // If non-zero, have been killed
};
